@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -48,6 +50,12 @@ Row<eT>::Row(const uword in_n_elem)
   : Mat<eT>(arma_vec_indicator(), 1, in_n_elem, 2)
   {
   arma_extra_debug_sigprint();
+  
+  if(arma_config::zero_init)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
   }
 
 
@@ -60,6 +68,12 @@ Row<eT>::Row(const uword in_n_rows, const uword in_n_cols)
   arma_extra_debug_sigprint();
   
   Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  
+  if(arma_config::zero_init)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
   }
 
 
@@ -72,6 +86,70 @@ Row<eT>::Row(const SizeMat& s)
   arma_extra_debug_sigprint();
   
   Mat<eT>::init_warm(s.n_rows, s.n_cols);
+  
+  if(arma_config::zero_init)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
+  }
+
+
+
+//! internal use only
+template<typename eT>
+template<bool do_zeros>
+inline
+Row<eT>::Row(const uword in_n_elem, const arma_initmode_indicator<do_zeros>&)
+  : Mat<eT>(arma_vec_indicator(), 1, in_n_elem, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(do_zeros)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
+  }
+
+
+
+//! internal use only
+template<typename eT>
+template<bool do_zeros>
+inline
+Row<eT>::Row(const uword in_n_rows, const uword in_n_cols, const arma_initmode_indicator<do_zeros>&)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  
+  if(do_zeros)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
+  }
+
+
+
+//! internal use only
+template<typename eT>
+template<bool do_zeros>
+inline
+Row<eT>::Row(const SizeMat& s, const arma_initmode_indicator<do_zeros>&)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::init_warm(s.n_rows, s.n_cols);
+  
+  if(do_zeros)
+    {
+    arma_extra_debug_print("Row::constructor: zeroing memory");
+    arrayops::fill_zeros(Mat<eT>::memptr(), Mat<eT>::n_elem);
+    }
   }
 
 
@@ -119,6 +197,46 @@ Row<eT>::Row(const SizeMat& s, const fill::fill_class<fill_type>& f)
 
 
 
+template<typename eT>
+inline
+Row<eT>::Row(const uword in_n_elem, const fill::scalar_holder<eT> f)
+  : Mat<eT>(arma_vec_indicator(), 1, in_n_elem, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  (*this).fill(f.scalar);
+  }
+
+
+
+template<typename eT>
+inline
+Row<eT>::Row(const uword in_n_rows, const uword in_n_cols, const fill::scalar_holder<eT> f)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::init_warm(in_n_rows, in_n_cols);
+  
+  (*this).fill(f.scalar);
+  }
+
+
+
+template<typename eT>
+inline
+Row<eT>::Row(const SizeMat& s, const fill::scalar_holder<eT> f)
+  : Mat<eT>(arma_vec_indicator(), 0, 0, 2)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT>::init_warm(s.n_rows, s.n_cols);
+  
+  (*this).fill(f.scalar);
+  }
+
+
+
 template<typename eT>
 inline
 Row<eT>::Row(const char* text)
@@ -193,10 +311,9 @@ Row<eT>::Row(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint_this(this);
   
-  if(x.size() > 0)
-    {
-    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
-    }
+  const uword N = uword(x.size());
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), &(x[0]), N ); }
   }
   
   
@@ -209,114 +326,160 @@ Row<eT>::operator=(const std::vector<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  Mat<eT>::init_warm(1, uword(x.size()));
+  const uword N = uword(x.size());
   
-  if(x.size() > 0)
-    {
-    arrayops::copy( Mat<eT>::memptr(), &(x[0]), uword(x.size()) );
-    }
+  Mat<eT>::init_warm(1, N);
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), &(x[0]), N ); }
   
   return *this;
   }
 
 
 
-#if defined(ARMA_USE_CXX11)
+template<typename eT>
+inline
+Row<eT>::Row(const std::initializer_list<eT>& list)
+  : Mat<eT>(arma_vec_indicator(), 1, uword(list.size()), 2)
+  {
+  arma_extra_debug_sigprint_this(this);
   
-  template<typename eT>
-  inline
-  Row<eT>::Row(const std::initializer_list<eT>& list)
-    : Mat<eT>(arma_vec_indicator(), 2)
+  const uword N = uword(list.size());
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), list.begin(), N ); }
+  }
+
+
+
+template<typename eT>
+inline
+Row<eT>&
+Row<eT>::operator=(const std::initializer_list<eT>& list)
+  {
+  arma_extra_debug_sigprint();
+  
+  const uword N = uword(list.size());
+  
+  Mat<eT>::init_warm(1, N);
+  
+  if(N > 0)  { arrayops::copy( Mat<eT>::memptr(), list.begin(), N ); }
+  
+  return *this;
+  }
+
+
+
+template<typename eT>
+inline
+Row<eT>::Row(Row<eT>&& X)
+  : Mat<eT>(arma_vec_indicator(), 2)
+  {
+  arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
+  
+  access::rw(Mat<eT>::n_rows)  = 1;
+  access::rw(Mat<eT>::n_cols)  = X.n_cols;
+  access::rw(Mat<eT>::n_elem)  = X.n_elem;
+  access::rw(Mat<eT>::n_alloc) = X.n_alloc;
+  
+  if( (X.n_alloc > arma_config::mat_prealloc) || (X.mem_state == 1) || (X.mem_state == 2) )
     {
-    arma_extra_debug_sigprint();
+    access::rw(Mat<eT>::mem_state) = X.mem_state;
+    access::rw(Mat<eT>::mem)       = X.mem;
     
-    (*this).operator=(list);
+    access::rw(X.n_rows)    = 1;
+    access::rw(X.n_cols)    = 0;
+    access::rw(X.n_elem)    = 0;
+    access::rw(X.n_alloc)   = 0;
+    access::rw(X.mem_state) = 0;
+    access::rw(X.mem)       = nullptr;
     }
-  
-  
-  
-  template<typename eT>
-  inline
-  Row<eT>&
-  Row<eT>::operator=(const std::initializer_list<eT>& list)
+  else  // condition: (X.n_alloc <= arma_config::mat_prealloc) || (X.mem_state == 0) || (X.mem_state == 3)
     {
-    arma_extra_debug_sigprint();
+    (*this).init_cold();
     
-    Mat<eT> tmp(list);
+    arrayops::copy( (*this).memptr(), X.mem, X.n_elem );
     
-    arma_debug_check( ((tmp.n_elem > 0) && (tmp.is_vec() == false)), "Mat::init(): requested size is not compatible with row vector layout" );
-    
-    access::rw(tmp.n_rows) = 1;
-    access::rw(tmp.n_cols) = tmp.n_elem;
-    
-    (*this).steal_mem(tmp);
-    
-    return *this;
-    }
-  
-  
-  
-  template<typename eT>
-  inline
-  Row<eT>::Row(Row<eT>&& X)
-    : Mat<eT>(arma_vec_indicator(), 2)
-    {
-    arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
-    
-    access::rw(Mat<eT>::n_rows) = 1;
-    access::rw(Mat<eT>::n_cols) = X.n_cols;
-    access::rw(Mat<eT>::n_elem) = X.n_elem;
-    
-    if( ((X.mem_state == 0) && (X.n_elem > arma_config::mat_prealloc)) || (X.mem_state == 1) || (X.mem_state == 2) )
+    if( (X.mem_state == 0) && (X.n_alloc <= arma_config::mat_prealloc) )
       {
-      access::rw(Mat<eT>::mem_state) = X.mem_state;
-      access::rw(Mat<eT>::mem)       = X.mem;
-      
-      access::rw(X.n_rows)    = 1;
-      access::rw(X.n_cols)    = 0;
-      access::rw(X.n_elem)    = 0;
-      access::rw(X.mem_state) = 0;
-      access::rw(X.mem)       = 0;
-      }
-    else
-      {
-      (*this).init_cold();
-      
-      arrayops::copy( (*this).memptr(), X.mem, X.n_elem );
-      
-      if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) )
-        {
-        access::rw(X.n_rows) = 1;
-        access::rw(X.n_cols) = 0;
-        access::rw(X.n_elem) = 0;
-        access::rw(X.mem)    = 0;
-        }
+      access::rw(X.n_rows)  = 1;
+      access::rw(X.n_cols)  = 0;
+      access::rw(X.n_elem)  = 0;
+      access::rw(X.mem)     = nullptr;
       }
     }
+  }
+
+
+
+template<typename eT>
+inline
+Row<eT>&
+Row<eT>::operator=(Row<eT>&& X)
+  {
+  arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
   
+  (*this).steal_mem(X, true);
   
-  
-  template<typename eT>
-  inline
-  Row<eT>&
-  Row<eT>::operator=(Row<eT>&& X)
-    {
-    arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
-    
-    (*this).steal_mem(X);
-    
-    if( (X.mem_state == 0) && (X.n_elem <= arma_config::mat_prealloc) && (this != &X) )
-      {
-      access::rw(X.n_rows) = 1;
-      access::rw(X.n_cols) = 0;
-      access::rw(X.n_elem) = 0;
-      access::rw(X.mem)    = 0;
-      }
-    
-    return *this;
-    }
-  
-#endif
+  return *this;
+  }
+
+
+
+// template<typename eT>
+// inline
+// Row<eT>::Row(Mat<eT>&& X)
+//   : Mat<eT>(arma_vec_indicator(), 2)
+//   {
+//   arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
+//   
+//   if(X.n_rows != 1)  { const Mat<eT>& XX = X; Mat<eT>::operator=(XX); return; }
+//   
+//   access::rw(Mat<eT>::n_rows)  = 1;
+//   access::rw(Mat<eT>::n_cols)  = X.n_cols;
+//   access::rw(Mat<eT>::n_elem)  = X.n_elem;
+//   access::rw(Mat<eT>::n_alloc) = X.n_alloc;
+//   
+//   if( (X.n_alloc > arma_config::mat_prealloc) || (X.mem_state == 1) || (X.mem_state == 2) )
+//     {
+//     access::rw(Mat<eT>::mem_state) = X.mem_state;
+//     access::rw(Mat<eT>::mem)       = X.mem;
+//     
+//     access::rw(X.n_cols)    = 0;
+//     access::rw(X.n_elem)    = 0;
+//     access::rw(X.n_alloc)   = 0;
+//     access::rw(X.mem_state) = 0;
+//     access::rw(X.mem)       = nullptr;
+//     }
+//   else  // condition: (X.n_alloc <= arma_config::mat_prealloc) || (X.mem_state == 0) || (X.mem_state == 3)
+//     {
+//     (*this).init_cold();
+//     
+//     arrayops::copy( (*this).memptr(), X.mem, X.n_elem );
+//     
+//     if( (X.mem_state == 0) && (X.n_alloc <= arma_config::mat_prealloc) )
+//       {
+//       access::rw(X.n_cols)  = 0;
+//       access::rw(X.n_elem)  = 0;
+//       access::rw(X.mem)     = nullptr;
+//       }
+//     }
+//   }
+// 
+// 
+// 
+// template<typename eT>
+// inline
+// Row<eT>&
+// Row<eT>::operator=(Mat<eT>&& X)
+//   {
+//   arma_extra_debug_sigprint(arma_str::format("this = %x   X = %x") % this % &X);
+//   
+//   if(X.n_rows != 1)  { const Mat<eT>& XX = X; Mat<eT>::operator=(XX); return *this; }
+//   
+//   (*this).steal_mem(X, true);
+//   
+//   return *this;
+//   }
 
 
 
@@ -561,7 +724,7 @@ Row<eT>::col(const uword in_col1)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used" );
   
   return subview_row<eT>(*this, 0, in_col1, 1);
   }
@@ -575,7 +738,7 @@ Row<eT>::col(const uword in_col1) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( (in_col1 >= Mat<eT>::n_cols), "Row::col(): indices out of bounds or incorrectly used" );
   
   return subview_row<eT>(*this, 0, in_col1, 1);
   }
@@ -589,7 +752,7 @@ Row<eT>::cols(const uword in_col1, const uword in_col2)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::cols(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::cols(): indices out of bounds or incorrectly used" );
   
   const uword subview_n_cols = in_col2 - in_col1 + 1;
   
@@ -605,7 +768,7 @@ Row<eT>::cols(const uword in_col1, const uword in_col2) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::cols(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::cols(): indices out of bounds or incorrectly used" );
   
   const uword subview_n_cols = in_col2 - in_col1 + 1;
   
@@ -621,7 +784,7 @@ Row<eT>::subvec(const uword in_col1, const uword in_col2)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::subvec(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::subvec(): indices out of bounds or incorrectly used" );
   
   const uword subview_n_cols = in_col2 - in_col1 + 1;
   
@@ -637,7 +800,7 @@ Row<eT>::subvec(const uword in_col1, const uword in_col2) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::subvec(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols) ), "Row::subvec(): indices out of bounds or incorrectly used" );
   
   const uword subview_n_cols = in_col2 - in_col1 + 1;
   
@@ -685,7 +848,7 @@ Row<eT>::subvec(const span& col_span)
   const uword in_col2       =                          col_span.b;
   const uword subvec_n_cols = col_all ? local_n_cols : in_col2 - in_col1 + 1;
 
-  arma_debug_check( ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) ), "Row::subvec(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) ), "Row::subvec(): indices out of bounds or incorrectly used" );
   
   return subview_row<eT>(*this, 0, in_col1, subvec_n_cols);
   }
@@ -707,7 +870,7 @@ Row<eT>::subvec(const span& col_span) const
   const uword in_col2       =                          col_span.b;
   const uword subvec_n_cols = col_all ? local_n_cols : in_col2 - in_col1 + 1;
 
-  arma_debug_check( ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) ), "Row::subvec(): indices out of bounds or incorrectly used");
+  arma_debug_check_bounds( ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) ), "Row::subvec(): indices out of bounds or incorrectly used" );
   
   return subview_row<eT>(*this, 0, in_col1, subvec_n_cols);
   }
@@ -747,7 +910,7 @@ Row<eT>::subvec(const uword start_col, const SizeMat& s)
   
   arma_debug_check( (s.n_rows != 1), "Row::subvec(): given size does not specify a row vector" );
   
-  arma_debug_check( ( (start_col >= Mat<eT>::n_cols) || ((start_col + s.n_cols) > Mat<eT>::n_cols) ), "Row::subvec(): size out of bounds" );
+  arma_debug_check_bounds( ( (start_col >= Mat<eT>::n_cols) || ((start_col + s.n_cols) > Mat<eT>::n_cols) ), "Row::subvec(): size out of bounds" );
   
   return subview_row<eT>(*this, 0, start_col, s.n_cols);
   }
@@ -763,7 +926,7 @@ Row<eT>::subvec(const uword start_col, const SizeMat& s) const
   
   arma_debug_check( (s.n_rows != 1), "Row::subvec(): given size does not specify a row vector" );
   
-  arma_debug_check( ( (start_col >= Mat<eT>::n_cols) || ((start_col + s.n_cols) > Mat<eT>::n_cols) ), "Row::subvec(): size out of bounds" );
+  arma_debug_check_bounds( ( (start_col >= Mat<eT>::n_cols) || ((start_col + s.n_cols) > Mat<eT>::n_cols) ), "Row::subvec(): size out of bounds" );
   
   return subview_row<eT>(*this, 0, start_col, s.n_cols);
   }
@@ -777,7 +940,7 @@ Row<eT>::head(const uword N)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (N > Mat<eT>::n_cols), "Row::head(): size out of bounds");
+  arma_debug_check_bounds( (N > Mat<eT>::n_cols), "Row::head(): size out of bounds" );
   
   return subview_row<eT>(*this, 0, 0, N);
   }
@@ -791,7 +954,7 @@ Row<eT>::head(const uword N) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (N > Mat<eT>::n_cols), "Row::head(): size out of bounds");
+  arma_debug_check_bounds( (N > Mat<eT>::n_cols), "Row::head(): size out of bounds" );
   
   return subview_row<eT>(*this, 0, 0, N);
   }
@@ -805,7 +968,7 @@ Row<eT>::tail(const uword N)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (N > Mat<eT>::n_cols), "Row::tail(): size out of bounds");
+  arma_debug_check_bounds( (N > Mat<eT>::n_cols), "Row::tail(): size out of bounds" );
   
   const uword start_col = Mat<eT>::n_cols - N;
   
@@ -821,7 +984,7 @@ Row<eT>::tail(const uword N) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (N > Mat<eT>::n_cols), "Row::tail(): size out of bounds");
+  arma_debug_check_bounds( (N > Mat<eT>::n_cols), "Row::tail(): size out of bounds" );
   
   const uword start_col = Mat<eT>::n_cols - N;
   
@@ -886,7 +1049,7 @@ Row<eT>::shed_col(const uword col_num)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( col_num >= Mat<eT>::n_cols, "Row::shed_col(): index out of bounds");
+  arma_debug_check_bounds( col_num >= Mat<eT>::n_cols, "Row::shed_col(): index out of bounds" );
   
   shed_cols(col_num, col_num);
   }
@@ -901,7 +1064,7 @@ Row<eT>::shed_cols(const uword in_col1, const uword in_col2)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check
+  arma_debug_check_bounds
     (
     (in_col1 > in_col2) || (in_col2 >= Mat<eT>::n_cols),
     "Row::shed_cols(): indices out of bounds or incorrectly used"
@@ -910,7 +1073,7 @@ Row<eT>::shed_cols(const uword in_col1, const uword in_col2)
   const uword n_keep_front = in_col1;
   const uword n_keep_back  = Mat<eT>::n_cols - (in_col2 + 1);
   
-  Row<eT> X(n_keep_front + n_keep_back);
+  Row<eT> X(n_keep_front + n_keep_back, arma_nozeros_indicator());
   
         eT* X_mem = X.memptr();
   const eT* t_mem = (*this).memptr();
@@ -944,8 +1107,6 @@ Row<eT>::shed_cols(const Base<uword, T1>& indices)
 
 
 
-//! insert N cols at the specified col position,
-//! optionally setting the elements of the inserted cols to zero
 template<typename eT>
 inline
 void
@@ -953,38 +1114,48 @@ Row<eT>::insert_cols(const uword col_num, const uword N, const bool set_to_zero)
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(set_to_zero);
+  
+  (*this).insert_cols(col_num, N);
+  }
+
+
+
+template<typename eT>
+inline
+void
+Row<eT>::insert_cols(const uword col_num, const uword N)
+  {
+  arma_extra_debug_sigprint();
+  
   const uword t_n_cols = Mat<eT>::n_cols;
   
   const uword A_n_cols = col_num;
   const uword B_n_cols = t_n_cols - col_num;
   
   // insertion at col_num == n_cols is in effect an append operation
-  arma_debug_check( (col_num > t_n_cols), "Row::insert_cols(): index out of bounds");
+  arma_debug_check_bounds( (col_num > t_n_cols), "Row::insert_cols(): index out of bounds" );
   
-  if(N > 0)
+  if(N == 0)  { return; }
+  
+  Row<eT> out(t_n_cols + N, arma_nozeros_indicator());
+  
+        eT* out_mem = out.memptr();
+  const eT*   t_mem = (*this).memptr();
+  
+  if(A_n_cols > 0)
     {
-    Row<eT> out(t_n_cols + N);
-    
-          eT* out_mem = out.memptr();
-    const eT*   t_mem = (*this).memptr();
-    
-    if(A_n_cols > 0)
-      {
-      arrayops::copy( out_mem, t_mem, A_n_cols );
-      }
-    
-    if(B_n_cols > 0)
-      {
-      arrayops::copy( &(out_mem[col_num + N]), &(t_mem[col_num]), B_n_cols );
-      }
-    
-    if(set_to_zero)
-      {
-      arrayops::inplace_set( &(out_mem[col_num]), eT(0), N );
-      }
-    
-    Mat<eT>::steal_mem(out);
+    arrayops::copy( out_mem, t_mem, A_n_cols );
     }
+  
+  if(B_n_cols > 0)
+    {
+    arrayops::copy( &(out_mem[col_num + N]), &(t_mem[col_num]), B_n_cols );
+    }
+  
+  arrayops::fill_zeros( &(out_mem[col_num]), N );
+  
+  Mat<eT>::steal_mem(out);
   }
 
 
@@ -1006,7 +1177,6 @@ Row<eT>::insert_cols(const uword col_num, const Base<eT,T1>& X)
 
 template<typename eT>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::at(const uword i)
   {
@@ -1017,7 +1187,6 @@ Row<eT>::at(const uword i)
 
 template<typename eT>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::at(const uword i) const
   {
@@ -1028,7 +1197,6 @@ Row<eT>::at(const uword i) const
 
 template<typename eT>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::at(const uword, const uword in_col)
   {
@@ -1039,7 +1207,6 @@ Row<eT>::at(const uword, const uword in_col)
 
 template<typename eT>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::at(const uword, const uword in_col) const
   {
@@ -1055,7 +1222,7 @@ Row<eT>::begin_row(const uword row_num)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Row::begin_row(): index out of bounds");
+  arma_debug_check_bounds( (row_num >= Mat<eT>::n_rows), "Row::begin_row(): index out of bounds" );
   
   return Mat<eT>::memptr();
   }
@@ -1069,7 +1236,7 @@ Row<eT>::begin_row(const uword row_num) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Row::begin_row(): index out of bounds");
+  arma_debug_check_bounds( (row_num >= Mat<eT>::n_rows), "Row::begin_row(): index out of bounds" );
   
   return Mat<eT>::memptr();
   }
@@ -1083,7 +1250,7 @@ Row<eT>::end_row(const uword row_num)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Row::end_row(): index out of bounds");
+  arma_debug_check_bounds( (row_num >= Mat<eT>::n_rows), "Row::end_row(): index out of bounds" );
   
   return Mat<eT>::memptr() + Mat<eT>::n_cols;
   }
@@ -1097,7 +1264,7 @@ Row<eT>::end_row(const uword row_num) const
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (row_num >= Mat<eT>::n_rows), "Row::end_row(): index out of bounds");
+  arma_debug_check_bounds( (row_num >= Mat<eT>::n_rows), "Row::end_row(): index out of bounds" );
   
   return Mat<eT>::memptr() + Mat<eT>::n_cols;
   }
@@ -1111,6 +1278,15 @@ Row<eT>::fixed<fixed_n_elem>::fixed()
   : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
   {
   arma_extra_debug_sigprint_this(this);
+  
+  if(arma_config::zero_init)
+    {
+    arma_extra_debug_print("Row::fixed::constructor: zeroing memory");
+    
+    eT* mem_use = (use_extra) ? &(mem_local_extra[0]) : &(Mat<eT>::mem_local[0]);
+    
+    arrayops::inplace_set_fixed<eT,fixed_n_elem>( mem_use, eT(0) );
+    }
   }
 
 
@@ -1144,6 +1320,19 @@ Row<eT>::fixed<fixed_n_elem>::fixed(const subview_cube<eT>& X)
 
 
 
+template<typename eT>
+template<uword fixed_n_elem>
+inline
+Row<eT>::fixed<fixed_n_elem>::fixed(const fill::scalar_holder<eT> f)
+  : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+  {
+  arma_extra_debug_sigprint_this(this);
+  
+  (*this).fill(f.scalar);
+  }
+
+
+
 template<typename eT>
 template<uword fixed_n_elem>
 template<typename fill_type>
@@ -1153,11 +1342,11 @@ Row<eT>::fixed<fixed_n_elem>::fixed(const fill::fill_class<fill_type>&)
   {
   arma_extra_debug_sigprint_this(this);
   
-  if(is_same_type<fill_type, fill::fill_zeros>::yes)  (*this).zeros();
-  if(is_same_type<fill_type, fill::fill_ones >::yes)  (*this).ones();
-  if(is_same_type<fill_type, fill::fill_eye  >::yes)  (*this).eye();
-  if(is_same_type<fill_type, fill::fill_randu>::yes)  (*this).randu();
-  if(is_same_type<fill_type, fill::fill_randn>::yes)  (*this).randn();
+  if(is_same_type<fill_type, fill::fill_zeros>::yes)  { (*this).zeros(); }
+  if(is_same_type<fill_type, fill::fill_ones >::yes)  { (*this).ones();  }
+  if(is_same_type<fill_type, fill::fill_eye  >::yes)  { (*this).eye();   }
+  if(is_same_type<fill_type, fill::fill_randu>::yes)  { (*this).randu(); }
+  if(is_same_type<fill_type, fill::fill_randn>::yes)  { (*this).randn(); }
   }
 
 
@@ -1302,43 +1491,39 @@ Row<eT>::fixed<fixed_n_elem>::operator=(const subview_cube<eT>& X)
 
 
 
-#if defined(ARMA_USE_CXX11)
+template<typename eT>
+template<uword fixed_n_elem>
+inline
+Row<eT>::fixed<fixed_n_elem>::fixed(const std::initializer_list<eT>& list)
+  : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
+  {
+  arma_extra_debug_sigprint_this(this);
   
-  template<typename eT>
-  template<uword fixed_n_elem>
-  inline
-  Row<eT>::fixed<fixed_n_elem>::fixed(const std::initializer_list<eT>& list)
-    : Row<eT>( arma_fixed_indicator(), fixed_n_elem, ((use_extra) ? mem_local_extra : Mat<eT>::mem_local) )
-    {
-    arma_extra_debug_sigprint_this(this);
-    
-    (*this).operator=(list);
-    }
+  (*this).operator=(list);
+  }
+
+
+
+template<typename eT>
+template<uword fixed_n_elem>
+inline
+Row<eT>&
+Row<eT>::fixed<fixed_n_elem>::operator=(const std::initializer_list<eT>& list)
+  {
+  arma_extra_debug_sigprint();
   
+  const uword N = uword(list.size());
   
+  arma_debug_check( (N > fixed_n_elem), "Row::fixed: initialiser list is too long" );
   
-  template<typename eT>
-  template<uword fixed_n_elem>
-  inline
-  Row<eT>&
-  Row<eT>::fixed<fixed_n_elem>::operator=(const std::initializer_list<eT>& list)
-    {
-    arma_extra_debug_sigprint();
-    
-    const uword N = uword(list.size());
-    
-    arma_debug_check( (N > fixed_n_elem), "Row::fixed: initialiser list is too long" );
-    
-    eT* this_mem = (*this).memptr();
-    
-    arrayops::copy( this_mem, list.begin(), N );
-    
-    for(uword iq=N; iq < fixed_n_elem; ++iq) { this_mem[iq] = eT(0); }
-    
-    return *this;
-    }
+  eT* this_mem = (*this).memptr();
   
-#endif
+  arrayops::copy( this_mem, list.begin(), N );
+  
+  for(uword iq=N; iq < fixed_n_elem; ++iq) { this_mem[iq] = eT(0); }
+  
+  return *this;
+  }
 
 
 
@@ -1475,7 +1660,6 @@ Row<eT>::fixed<fixed_n_elem>::st() const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::at_alt(const uword ii) const
   {
@@ -1497,7 +1681,6 @@ Row<eT>::fixed<fixed_n_elem>::at_alt(const uword ii) const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::operator[] (const uword ii)
   {
@@ -1509,7 +1692,6 @@ Row<eT>::fixed<fixed_n_elem>::operator[] (const uword ii)
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::operator[] (const uword ii) const
   {
@@ -1521,7 +1703,6 @@ Row<eT>::fixed<fixed_n_elem>::operator[] (const uword ii) const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword ii)
   {
@@ -1533,7 +1714,6 @@ Row<eT>::fixed<fixed_n_elem>::at(const uword ii)
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword ii) const
   {
@@ -1545,11 +1725,10 @@ Row<eT>::fixed<fixed_n_elem>::at(const uword ii) const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii)
   {
-  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): index out of bounds");
+  arma_debug_check_bounds( (ii >= fixed_n_elem), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
@@ -1559,11 +1738,10 @@ Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii)
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii) const
   {
-  arma_debug_check( (ii >= fixed_n_elem), "Row::operator(): index out of bounds");
+  arma_debug_check_bounds( (ii >= fixed_n_elem), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[ii] : Mat<eT>::mem_local[ii];
   }
@@ -1573,7 +1751,6 @@ Row<eT>::fixed<fixed_n_elem>::operator() (const uword ii) const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword, const uword in_col)
   {
@@ -1585,7 +1762,6 @@ Row<eT>::fixed<fixed_n_elem>::at(const uword, const uword in_col)
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::at(const uword, const uword in_col) const
   {
@@ -1597,11 +1773,10 @@ Row<eT>::fixed<fixed_n_elem>::at(const uword, const uword in_col) const
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col)
   {
-  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
+  arma_debug_check_bounds( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[in_col] : Mat<eT>::mem_local[in_col];
   }
@@ -1611,11 +1786,10 @@ Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT&
 Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col) const
   {
-  arma_debug_check( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
+  arma_debug_check_bounds( ((in_row > 0) || (in_col >= fixed_n_elem)), "Row::operator(): index out of bounds" );
   
   return (use_extra) ? mem_local_extra[in_col] : Mat<eT>::mem_local[in_col];
   }
@@ -1625,7 +1799,6 @@ Row<eT>::fixed<fixed_n_elem>::operator() (const uword in_row, const uword in_col
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 eT*
 Row<eT>::fixed<fixed_n_elem>::memptr()
   {
@@ -1637,7 +1810,6 @@ Row<eT>::fixed<fixed_n_elem>::memptr()
 template<typename eT>
 template<uword fixed_n_elem>
 arma_inline
-arma_warn_unused
 const eT*
 Row<eT>::fixed<fixed_n_elem>::memptr() const
   {
@@ -1648,7 +1820,6 @@ Row<eT>::fixed<fixed_n_elem>::memptr() const
 
 template<typename eT>
 template<uword fixed_n_elem>
-arma_hot
 inline
 const Row<eT>&
 Row<eT>::fixed<fixed_n_elem>::fill(const eT val)
@@ -1666,7 +1837,6 @@ Row<eT>::fixed<fixed_n_elem>::fill(const eT val)
 
 template<typename eT>
 template<uword fixed_n_elem>
-arma_hot
 inline
 const Row<eT>&
 Row<eT>::fixed<fixed_n_elem>::zeros()
@@ -1684,7 +1854,6 @@ Row<eT>::fixed<fixed_n_elem>::zeros()
 
 template<typename eT>
 template<uword fixed_n_elem>
-arma_hot
 inline
 const Row<eT>&
 Row<eT>::fixed<fixed_n_elem>::ones()
@@ -1710,7 +1879,7 @@ Row<eT>::Row(const arma_fixed_indicator&, const uword in_n_elem, const eT* in_me
 
 
 
-#ifdef ARMA_EXTRA_ROW_MEAT
+#if defined(ARMA_EXTRA_ROW_MEAT)
   #include ARMA_INCFILE_WRAP(ARMA_EXTRA_ROW_MEAT)
 #endif
 
