@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -69,109 +71,235 @@ SpBase<elem_type,derived>::st() const
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type,derived>::print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, true);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type,derived>::print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, true);
   }
   
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type,derived>::raw_print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, false);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type,derived>::raw_print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, false);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type, derived>::print_dense(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
 
-  tmp.M.impl_print_dense(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print_dense(get_cout_stream(), tmp.M, true);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type, derived>::print_dense(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
 
-  tmp.M.impl_print_dense(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print_dense(user_stream, tmp.M, true);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type, derived>::raw_print_dense(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
 
-  tmp.M.impl_raw_print_dense(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print_dense(get_cout_stream(), tmp.M, false);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 SpBase<elem_type, derived>::raw_print_dense(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const unwrap_spmat<derived> tmp( (*this).get_ref() );
 
-  tmp.M.impl_raw_print_dense(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print_dense(user_stream, tmp.M, false);
   }
 
 
 
+template<typename elem_type, typename derived>
+inline
+void
+SpBase<elem_type,derived>::brief_print(const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap_spmat<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::brief_print(get_cout_stream(), tmp.M);
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+void
+SpBase<elem_type,derived>::brief_print(std::ostream& user_stream, const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap_spmat<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::brief_print(user_stream, tmp.M);
+  }
+  
+
+
 //
 // extra functions defined in SpBase_eval_SpMat
 
@@ -204,7 +332,6 @@ SpBase_eval_expr<elem_type, derived>::eval() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 elem_type
 SpBase<elem_type, derived>::min() const
   {
@@ -215,7 +342,6 @@ SpBase<elem_type, derived>::min() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 elem_type
 SpBase<elem_type, derived>::max() const
   {
@@ -292,7 +418,6 @@ SpBase<elem_type, derived>::max(uword& row_of_max_val, uword& col_of_max_val) co
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 uword
 SpBase<elem_type,derived>::index_min() const
   {
@@ -316,7 +441,6 @@ SpBase<elem_type,derived>::index_min() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 uword
 SpBase<elem_type,derived>::index_max() const
   {
@@ -340,7 +464,6 @@ SpBase<elem_type,derived>::index_max() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_symmetric() const
   {
@@ -355,7 +478,6 @@ SpBase<elem_type,derived>::is_symmetric() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_symmetric(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -370,7 +492,6 @@ SpBase<elem_type,derived>::is_symmetric(const typename get_pod_type<elem_type>::
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_hermitian() const
   {
@@ -385,7 +506,6 @@ SpBase<elem_type,derived>::is_hermitian() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -400,7 +520,6 @@ SpBase<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -435,8 +554,8 @@ SpBase<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::resul
       const T val_real = access::tmp_real(val);
       const T val_imag = access::tmp_imag(val);
       
-      if(std::abs(val_real) > tol)  { return false; }
-      if(std::abs(val_imag) > tol)  { return false; }
+      if(eop_aux::arma_abs(val_real) > tol)  { return false; }
+      if(eop_aux::arma_abs(val_imag) > tol)  { return false; }
       
       ++it;
       }
@@ -445,7 +564,7 @@ SpBase<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::resul
     {
     while(it != it_end)
       {
-      if(std::abs(*it) > tol)  { return false; }
+      if(eop_aux::arma_abs(*it) > tol)  { return false; }
       
       ++it;
       }
@@ -458,7 +577,6 @@ SpBase<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::resul
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_trimatu() const
   {
@@ -484,7 +602,6 @@ SpBase<elem_type,derived>::is_trimatu() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_trimatl() const
   {
@@ -510,7 +627,6 @@ SpBase<elem_type,derived>::is_trimatl() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_diagmat() const
   {
@@ -534,7 +650,6 @@ SpBase<elem_type,derived>::is_diagmat() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_empty() const
   {
@@ -549,7 +664,6 @@ SpBase<elem_type,derived>::is_empty() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_square() const
   {
@@ -564,7 +678,6 @@ SpBase<elem_type,derived>::is_square() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_vec() const
   {
@@ -581,7 +694,6 @@ SpBase<elem_type,derived>::is_vec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_colvec() const
   {
@@ -598,7 +710,6 @@ SpBase<elem_type,derived>::is_colvec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_rowvec() const
   {
@@ -615,22 +726,23 @@ SpBase<elem_type,derived>::is_rowvec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::is_finite() const
   {
   arma_extra_debug_sigprint();
   
-  const SpProxy<derived> P( (*this).get_ref() );
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "is_finite(): detection of non-finite values is not reliable in fast math mode"); }
   
   if(is_SpMat<typename SpProxy<derived>::stored_type>::value)
     {
-    const unwrap_spmat<typename SpProxy<derived>::stored_type> U(P.Q);
+    const unwrap_spmat<derived> U( (*this).get_ref() );
     
-    return U.M.is_finite();
+    return U.M.internal_is_finite();
     }
   else
     {
+    const SpProxy<derived> P( (*this).get_ref() );
+    
     typename SpProxy<derived>::const_iterator_type it     = P.begin();
     typename SpProxy<derived>::const_iterator_type it_end = P.end();
     
@@ -648,22 +760,23 @@ SpBase<elem_type,derived>::is_finite() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::has_inf() const
   {
   arma_extra_debug_sigprint();
   
-  const SpProxy<derived> P( (*this).get_ref() );
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_inf(): detection of non-finite values is not reliable in fast math mode"); }
   
   if(is_SpMat<typename SpProxy<derived>::stored_type>::value)
     {
-    const unwrap_spmat<typename SpProxy<derived>::stored_type> U(P.Q);
+    const unwrap_spmat<derived> U( (*this).get_ref() );
     
-    return U.M.has_inf();
+    return U.M.internal_has_inf();
     }
   else
     {
+    const SpProxy<derived> P( (*this).get_ref() );
+    
     typename SpProxy<derived>::const_iterator_type it     = P.begin();
     typename SpProxy<derived>::const_iterator_type it_end = P.end();
     
@@ -681,22 +794,23 @@ SpBase<elem_type,derived>::has_inf() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 SpBase<elem_type,derived>::has_nan() const
   {
   arma_extra_debug_sigprint();
   
-  const SpProxy<derived> P( (*this).get_ref() );
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_nan(): detection of non-finite values is not reliable in fast math mode"); }
   
   if(is_SpMat<typename SpProxy<derived>::stored_type>::value)
     {
-    const unwrap_spmat<typename SpProxy<derived>::stored_type> U(P.Q);
+    const unwrap_spmat<derived> U( (*this).get_ref() );
     
-    return U.M.has_nan();
+    return U.M.internal_has_nan();
     }
   else
     {
+    const SpProxy<derived> P( (*this).get_ref() );
+    
     typename SpProxy<derived>::const_iterator_type it     = P.begin();
     typename SpProxy<derived>::const_iterator_type it_end = P.end();
     
@@ -712,6 +826,40 @@ SpBase<elem_type,derived>::has_nan() const
 
 
 
+template<typename elem_type, typename derived>
+inline
+bool
+SpBase<elem_type,derived>::has_nonfinite() const
+  {
+  arma_extra_debug_sigprint();
+  
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_nonfinite(): detection of non-finite values is not reliable in fast math mode"); }
+  
+  if(is_SpMat<typename SpProxy<derived>::stored_type>::value)
+    {
+    const unwrap_spmat<derived> U( (*this).get_ref() );
+    
+    return U.M.internal_has_nonfinite();
+    }
+  else
+    {
+    const SpProxy<derived> P( (*this).get_ref() );
+    
+    typename SpProxy<derived>::const_iterator_type it     = P.begin();
+    typename SpProxy<derived>::const_iterator_type it_end = P.end();
+    
+    while(it != it_end)
+      {
+      if(arma_isfinite(*it) == false)  { return true; }
+      ++it;
+      }
+    }
+  
+  return false;
+  }
+
+
+
 template<typename elem_type, typename derived>
 inline
 const SpOp<derived,spop_vectorise_col>
