@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -19,57 +21,11 @@
 
 
 
-template<typename T>
-inline
-std::ostream&
-arma_cout_stream(std::ostream* user_stream)
-  {
-  static std::ostream* cout_stream = &(ARMA_COUT_STREAM);
-  
-  if(user_stream != NULL)  { cout_stream = user_stream; }
-  
-  return (*cout_stream);
-  }
-
-
-
-template<typename T>
-inline
-std::ostream&
-arma_cerr_stream(std::ostream* user_stream)
-  {
-  static std::ostream* cerr_stream = &(ARMA_CERR_STREAM);
-  
-  if(user_stream != NULL)  { cerr_stream = user_stream; }
-  
-  return (*cerr_stream);
-  }
-
-
-
-inline
-void
-set_cout_stream(std::ostream& user_stream)
-  {
-  arma_cout_stream<char>(&user_stream);
-  }
-
-
-
-inline
-void
-set_cerr_stream(std::ostream& user_stream)
-  {
-  arma_cerr_stream<char>(&user_stream);
-  }
-
-
-
 inline
 std::ostream&
 get_cout_stream()
   {
-  return arma_cout_stream<char>(NULL);
+  return (ARMA_COUT_STREAM);
   }
 
 
@@ -78,36 +34,13 @@ inline
 std::ostream&
 get_cerr_stream()
   {
-  return arma_cerr_stream<char>(NULL);
+  return (ARMA_CERR_STREAM);
   }
 
 
 
-//! do not use this function - it's deprecated and will be removed
-inline
 arma_deprecated
-void
-set_stream_err1(std::ostream& user_stream)
-  {
-  set_cerr_stream(user_stream);
-  }
-
-
-
-//! do not use this function - it's deprecated and will be removed
 inline
-arma_deprecated
-void
-set_stream_err2(std::ostream& user_stream)
-  {
-  set_cerr_stream(user_stream);
-  }
-
-
-
-//! do not use this function - it's deprecated and will be removed
-inline
-arma_deprecated
 std::ostream&
 get_stream_err1()
   {
@@ -116,9 +49,8 @@ get_stream_err1()
 
 
 
-//! do not use this function - it's deprecated and will be removed
-inline
 arma_deprecated
+inline
 std::ostream&
 get_stream_err2()
   {
@@ -127,6 +59,64 @@ get_stream_err2()
 
 
 
+arma_frown("this function does nothing; instead use ARMA_COUT_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+void
+set_cout_stream(const std::ostream&)
+  {
+  }
+
+
+
+arma_frown("this function does nothing; instead use ARMA_CERR_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+void
+set_cerr_stream(const std::ostream&)
+  {
+  }
+
+
+
+arma_frown("this function does nothing; instead use ARMA_CERR_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+void
+set_stream_err1(const std::ostream&)
+  {
+  }
+
+
+
+arma_frown("this function does nothing; instead use ARMA_CERR_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+void
+set_stream_err2(const std::ostream&)
+  {
+  }
+
+
+
+template<typename T>
+arma_frown("this function does nothing; instead use ARMA_COUT_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+std::ostream&
+arma_cout_stream(std::ostream*)
+  {
+  return (ARMA_COUT_STREAM);
+  }
+
+
+
+template<typename T>
+arma_frown("this function does nothing; instead use ARMA_CERR_STREAM or ARMA_WARN_LEVEL; see documentation")
+inline
+std::ostream&
+arma_cerr_stream(std::ostream*)
+  {
+  return (ARMA_CERR_STREAM);
+  }
+
+
+
 //! print a message to get_cerr_stream() and throw logic_error exception
 template<typename T1>
 arma_cold
@@ -135,7 +125,7 @@ static
 void
 arma_stop_logic_error(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if defined(ARMA_PRINT_EXCEPTIONS)
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -146,6 +136,36 @@ arma_stop_logic_error(const T1& x)
 
 
 
+arma_cold
+arma_noinline
+static
+void
+arma_stop_logic_error(const char* x, const char* y)
+  {
+  arma_stop_logic_error( std::string(x) + std::string(y) );
+  }
+
+
+
+//! print a message to get_cerr_stream() and throw out_of_range exception
+template<typename T1>
+arma_cold
+arma_noinline
+static
+void
+arma_stop_bounds_error(const T1& x)
+  {
+  #if defined(ARMA_PRINT_EXCEPTIONS)
+    {
+    get_cerr_stream() << "\nerror: " << x << std::endl;
+    }
+  #endif
+  
+  throw std::out_of_range( std::string(x) );
+  }
+
+
+
 //! print a message to get_cerr_stream() and throw bad_alloc exception
 template<typename T1>
 arma_cold
@@ -154,7 +174,7 @@ static
 void
 arma_stop_bad_alloc(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if defined(ARMA_PRINT_EXCEPTIONS)
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -177,7 +197,7 @@ static
 void
 arma_stop_runtime_error(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if defined(ARMA_PRINT_EXCEPTIONS)
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -243,7 +263,7 @@ arma_print(const T1& x, const T2& y, const T3& z)
 //
 // arma_sigprint
 
-//! print a message the the log stream with a preceding @ character.
+//! print a message to the log stream with a preceding @ character.
 //! by default the log stream is cout.
 //! used for printing the signature of a function
 //! (see the arma_extra_debug_sigprint macro) 
@@ -313,17 +333,9 @@ arma_cold
 arma_noinline
 static
 void
-arma_warn(const T1& x)
+arma_warn(const T1& arg1)
   {
-  #if defined(ARMA_PRINT_ERRORS)
-    {
-    get_cerr_stream() << "\nwarning: " << x << '\n';
-    }
-  #else
-    {
-    arma_ignore(x);
-    }
-  #endif
+  get_cerr_stream() << "\nwarning: " << arg1 << std::endl;
   }
 
 
@@ -332,18 +344,9 @@ arma_cold
 arma_noinline
 static
 void
-arma_warn(const T1& x, const T2& y)
+arma_warn(const T1& arg1, const T2& arg2)
   {
-  #if defined(ARMA_PRINT_ERRORS)
-    {
-    get_cerr_stream() << "\nwarning: " << x << y << '\n';
-    }
-  #else
-    {
-    arma_ignore(x);
-    arma_ignore(y);
-    }
-  #endif
+  get_cerr_stream() << "\nwarning: " << arg1 << arg2 << std::endl;
   }
 
 
@@ -352,19 +355,69 @@ arma_cold
 arma_noinline
 static
 void
-arma_warn(const T1& x, const T2& y, const T3& z)
+arma_warn(const T1& arg1, const T2& arg2, const T3& arg3)
   {
-  #if defined(ARMA_PRINT_ERRORS)
-    {
-    get_cerr_stream() << "\nwarning: " << x << y << z << '\n';
-    }
-  #else
-    {
-    arma_ignore(x);
-    arma_ignore(y);
-    arma_ignore(z);
-    }
-  #endif
+  get_cerr_stream() << "\nwarning: " << arg1 << arg2 << arg3 << std::endl;
+  }
+
+
+template<typename T1, typename T2, typename T3, typename T4>
+arma_cold
+arma_noinline
+static
+void
+arma_warn(const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4)
+  {
+  get_cerr_stream() << "\nwarning: " << arg1 << arg2 << arg3 << arg4 << std::endl;
+  }
+
+
+
+//
+// arma_warn_level
+
+
+template<typename T1>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1); }
+  }
+
+
+template<typename T1, typename T2>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2); }
+  }
+
+
+template<typename T1, typename T2, typename T3>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2, const T3& arg3)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2,arg3); }
+  }
+
+
+template<typename T1, typename T2, typename T3, typename T4>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2,arg3,arg4); }
   }
 
 
@@ -383,13 +436,42 @@ arma_check(const bool state, const T1& x)
   }
 
 
-template<typename T1, typename T2>
+template<typename Functor>
 arma_hot
 inline
 void
-arma_check(const bool state, const T1& x, const T2& y)
+arma_check(const bool state, const char* x, const Functor& fn)
   {
-  if(state)  { arma_stop_logic_error( std::string(x) + std::string(y) ); }
+  if(state)  { fn(); arma_stop_logic_error(x); }
+  }
+
+
+arma_hot
+inline
+void
+arma_check(const bool state, const char* x, const char* y)
+  {
+  if(state)  { arma_stop_logic_error(x,y); }
+  }
+
+
+template<typename Functor>
+arma_hot
+inline
+void
+arma_check(const bool state, const char* x, const char* y, const Functor& fn)
+  {
+  if(state)  { fn(); arma_stop_logic_error(x,y); }
+  }
+
+
+template<typename T1>
+arma_hot
+inline
+void
+arma_check_bounds(const bool state, const T1& x)
+  {
+  if(state)  { arma_stop_bounds_error(arma_str::str_wrapper(x)); }
   }
 
 
@@ -485,8 +567,8 @@ arma_incompat_size_string(const subview_cube<eT>& Q, const Mat<eT>& A, const cha
 
 
 
-arma_inline
 arma_hot
+arma_inline
 void
 arma_assert_same_size(const uword A_n_rows, const uword A_n_cols, const uword B_n_rows, const uword B_n_cols, const char* x)
   {
@@ -780,6 +862,28 @@ arma_assert_same_size(const subview_cube<eT1>& A, const subview_cube<eT2>& B, co
 
 
 
+template<typename eT, typename T1>
+arma_hot
+inline
+void
+arma_assert_same_size(const subview_cube<eT>& A, const ProxyCube<T1>& B, const char* x)
+  {
+  const uword A_n_rows   = A.n_rows;
+  const uword A_n_cols   = A.n_cols;
+  const uword A_n_slices = A.n_slices;
+  
+  const uword B_n_rows   = B.get_n_rows();
+  const uword B_n_cols   = B.get_n_cols();
+  const uword B_n_slices = B.get_n_slices();
+  
+  if( (A_n_rows != B_n_rows) || (A_n_cols != B_n_cols) || (A_n_slices != B_n_slices) )
+    {
+    arma_stop_logic_error( arma_incompat_size_string(A_n_rows, A_n_cols, A_n_slices, B_n_rows, B_n_cols, B_n_slices, x) );
+    }
+  }
+
+
+
 //! stop if given cube proxies have different sizes
 template<typename eT1, typename eT2>
 arma_hot
@@ -804,7 +908,7 @@ arma_assert_same_size(const ProxyCube<eT1>& A, const ProxyCube<eT2>& B, const ch
 
 
 //
-// functions for checking whether a cube or subcube can be interpreted as a matrix (i.e. single slice)
+// functions for checking whether a cube or subcube can be interpreted as a matrix (ie. single slice)
 
 
 
@@ -1179,6 +1283,7 @@ arma_assert_blas_size(const T1& A, const T2& B)
 
 
 
+// TODO: remove support for ATLAS in next major version
 template<typename T1>
 arma_hot
 inline
@@ -1201,6 +1306,7 @@ arma_assert_atlas_size(const T1& A)
 
 
 
+// TODO: remove support for ATLAS in next major version
 template<typename T1, typename T2>
 arma_hot
 inline
@@ -1236,11 +1342,11 @@ arma_assert_atlas_size(const T1& A, const T2& B)
 
 #if defined(ARMA_NO_DEBUG)
   
-  #undef ARMA_EXTRA_DEBUG
-  
   #define arma_debug_print                   true ? (void)0 : arma_print
   #define arma_debug_warn                    true ? (void)0 : arma_warn
+  #define arma_debug_warn_level              true ? (void)0 : arma_warn_level
   #define arma_debug_check                   true ? (void)0 : arma_check
+  #define arma_debug_check_bounds            true ? (void)0 : arma_check_bounds
   #define arma_debug_set_error               true ? (void)0 : arma_set_error
   #define arma_debug_assert_same_size        true ? (void)0 : arma_assert_same_size
   #define arma_debug_assert_mul_size         true ? (void)0 : arma_assert_mul_size
@@ -1253,7 +1359,9 @@ arma_assert_atlas_size(const T1& A, const T2& B)
   
   #define arma_debug_print                 arma_print
   #define arma_debug_warn                  arma_warn
+  #define arma_debug_warn_level            arma_warn_level
   #define arma_debug_check                 arma_check
+  #define arma_debug_check_bounds          arma_check_bounds
   #define arma_debug_set_error             arma_set_error
   #define arma_debug_assert_same_size      arma_assert_same_size
   #define arma_debug_assert_mul_size       arma_assert_mul_size
@@ -1271,17 +1379,13 @@ arma_assert_atlas_size(const T1& A, const T2& B)
   #define arma_extra_debug_sigprint       arma_sigprint(ARMA_FNSIG); arma_bktprint
   #define arma_extra_debug_sigprint_this  arma_sigprint(ARMA_FNSIG); arma_thisprint
   #define arma_extra_debug_print          arma_print
-  #define arma_extra_debug_warn           arma_warn
-  #define arma_extra_debug_check          arma_check
-
+  
 #else
   
   #define arma_extra_debug_sigprint        true ? (void)0 : arma_bktprint
   #define arma_extra_debug_sigprint_this   true ? (void)0 : arma_thisprint
   #define arma_extra_debug_print           true ? (void)0 : arma_print
-  #define arma_extra_debug_warn            true ? (void)0 : arma_warn
-  #define arma_extra_debug_check           true ? (void)0 : arma_check
- 
+  
 #endif
 
 
@@ -1316,24 +1420,32 @@ arma_assert_atlas_size(const T1& A, const T2& B)
             << arma_version::major << '.' << arma_version::minor << '.' << arma_version::patch
             << " (" << nickname << ")\n";
         
-        out << "@ arma_config::wrapper      = " << arma_config::wrapper      << '\n';
-        out << "@ arma_config::cxx11        = " << arma_config::cxx11        << '\n';
-        out << "@ arma_config::cxx11_mutex  = " << arma_config::cxx11_mutex  << '\n';
-        out << "@ arma_config::posix        = " << arma_config::posix        << '\n';
-        out << "@ arma_config::openmp       = " << arma_config::openmp       << '\n';
-        out << "@ arma_config::lapack       = " << arma_config::lapack       << '\n';
-        out << "@ arma_config::blas         = " << arma_config::blas         << '\n';
-        out << "@ arma_config::newarp       = " << arma_config::newarp       << '\n';
-        out << "@ arma_config::arpack       = " << arma_config::arpack       << '\n';
-        out << "@ arma_config::superlu      = " << arma_config::superlu      << '\n';
-        out << "@ arma_config::atlas        = " << arma_config::atlas        << '\n';
-        out << "@ arma_config::hdf5         = " << arma_config::hdf5         << '\n';
-        out << "@ arma_config::good_comp    = " << arma_config::good_comp    << '\n';
-        out << "@ arma_config::extra_code   = " << arma_config::extra_code   << '\n';
-        out << "@ arma_config::hidden_args  = " << arma_config::hidden_args  << '\n';
-        out << "@ arma_config::mat_prealloc = " << arma_config::mat_prealloc << '\n';
-        out << "@ arma_config::mp_threshold = " << arma_config::mp_threshold << '\n';
-        out << "@ arma_config::mp_threads   = " << arma_config::mp_threads   << '\n';
+        out << "@ arma_config::wrapper          = " << arma_config::wrapper          << '\n';
+        out << "@ arma_config::cxx14            = " << arma_config::cxx14            << '\n';
+        out << "@ arma_config::cxx17            = " << arma_config::cxx17            << '\n';
+        out << "@ arma_config::cxx20            = " << arma_config::cxx20            << '\n';
+        out << "@ arma_config::std_mutex        = " << arma_config::std_mutex        << '\n';
+        out << "@ arma_config::posix            = " << arma_config::posix            << '\n';
+        out << "@ arma_config::openmp           = " << arma_config::openmp           << '\n';
+        out << "@ arma_config::lapack           = " << arma_config::lapack           << '\n';
+        out << "@ arma_config::blas             = " << arma_config::blas             << '\n';
+        out << "@ arma_config::newarp           = " << arma_config::newarp           << '\n';
+        out << "@ arma_config::arpack           = " << arma_config::arpack           << '\n';
+        out << "@ arma_config::superlu          = " << arma_config::superlu          << '\n';
+        out << "@ arma_config::atlas            = " << arma_config::atlas            << '\n';
+        out << "@ arma_config::hdf5             = " << arma_config::hdf5             << '\n';
+        out << "@ arma_config::good_comp        = " << arma_config::good_comp        << '\n';
+        out << "@ arma_config::extra_code       = " << arma_config::extra_code       << '\n';
+        out << "@ arma_config::hidden_args      = " << arma_config::hidden_args      << '\n';
+        out << "@ arma_config::mat_prealloc     = " << arma_config::mat_prealloc     << '\n';
+        out << "@ arma_config::mp_threshold     = " << arma_config::mp_threshold     << '\n';
+        out << "@ arma_config::mp_threads       = " << arma_config::mp_threads       << '\n';
+        out << "@ arma_config::optimise_band    = " << arma_config::optimise_band    << '\n';
+        out << "@ arma_config::optimise_sym     = " << arma_config::optimise_sym     << '\n';
+        out << "@ arma_config::optimise_invexpr = " << arma_config::optimise_invexpr << '\n';
+        out << "@ arma_config::check_nonfinite  = " << arma_config::check_nonfinite  << '\n';
+        out << "@ arma_config::zero_init        = " << arma_config::zero_init        << '\n';
+        out << "@ arma_config::fast_math        = " << arma_config::fast_math        << '\n';
         out << "@ sizeof(void*)    = " << sizeof(void*)    << '\n';
         out << "@ sizeof(int)      = " << sizeof(int)      << '\n';
         out << "@ sizeof(long)     = " << sizeof(long)     << '\n';
