@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -28,8 +30,11 @@ class op_powmat
   template<typename T1>
   inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_powmat>& expr);
   
+  template<typename T1>
+  inline static bool apply_direct(Mat<typename T1::elem_type>& out, const Base<typename T1::elem_type,T1>& X, const uword y, const bool y_neg);
+  
   template<typename eT>
-  inline static void apply(Mat<eT>& out, const Mat<eT>& X, const uword y);
+  inline static void apply_direct_positive(Mat<eT>& out, const Mat<eT>& X, const uword y);
   };
 
 
@@ -41,6 +46,9 @@ class op_powmat_cx
   
   template<typename T1>
   inline static void apply(Mat< std::complex<typename T1::pod_type> >& out, const mtOp<std::complex<typename T1::pod_type>,T1,op_powmat_cx>& expr);
+  
+  template<typename T1>
+  inline static bool apply_direct(Mat< std::complex<typename T1::pod_type> >& out, const Base<typename T1::elem_type,T1>& X, const typename T1::pod_type y);
   };
 
 
