@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -31,7 +33,7 @@ struct strip_diagmat
     arma_extra_debug_sigprint();
     }
   
-  static const bool do_diagmat = false;
+  static constexpr bool do_diagmat = false;
   
   const T1& M;
   };
@@ -50,7 +52,7 @@ struct strip_diagmat< Op<T1, op_diagmat> >
     arma_extra_debug_sigprint();
     }
   
-  static const bool do_diagmat = true;
+  static constexpr bool do_diagmat = true;
   
   const T1& M;
   };
@@ -71,19 +73,19 @@ struct strip_inv
   
   const T1& M;
   
-  static const bool do_inv       = false;
-  static const bool do_inv_sympd = false;
+  static constexpr bool do_inv_gen = false;
+  static constexpr bool do_inv_spd = false;
   };
 
 
 
 template<typename T1>
-struct strip_inv< Op<T1, op_inv> >
+struct strip_inv< Op<T1, op_inv_gen_default> >
   {
   typedef T1 stored_type;
   
   inline
-  strip_inv(const Op<T1, op_inv>& X)
+  strip_inv(const Op<T1, op_inv_gen_default>& X)
     : M(X.m)
     {
     arma_extra_debug_sigprint();
@@ -91,19 +93,19 @@ struct strip_inv< Op<T1, op_inv> >
   
   const T1& M;
   
-  static const bool do_inv       = true;
-  static const bool do_inv_sympd = false;
+  static constexpr bool do_inv_gen = true;
+  static constexpr bool do_inv_spd = false;
   };
 
 
 
 template<typename T1>
-struct strip_inv< Op<T1, op_inv_sympd> >
+struct strip_inv< Op<T1, op_inv_spd_default> >
   {
   typedef T1 stored_type;
   
   inline
-  strip_inv(const Op<T1, op_inv_sympd>& X)
+  strip_inv(const Op<T1, op_inv_spd_default>& X)
     : M(X.m)
     {
     arma_extra_debug_sigprint();
@@ -111,8 +113,8 @@ struct strip_inv< Op<T1, op_inv_sympd> >
   
   const T1& M;
   
-  static const bool do_inv       = true;
-  static const bool do_inv_sympd = true;
+  static constexpr bool do_inv_gen = false;
+  static constexpr bool do_inv_spd = true;
   };
 
 
@@ -124,9 +126,9 @@ struct strip_trimat
   
   const T1& M;
   
-  static const bool do_trimat = false;
-  static const bool do_triu   = false;
-  static const bool do_tril   = false;
+  static constexpr bool do_trimat = false;
+  static constexpr bool do_triu   = false;
+  static constexpr bool do_tril   = false;
   
   inline
   strip_trimat(const T1& X)
@@ -145,9 +147,10 @@ struct strip_trimat< Op<T1, op_trimat> >
   
   const T1& M;
   
-  static const bool do_trimat = true;
-         const bool do_triu;
-         const bool do_tril;
+  static constexpr bool do_trimat = true;
+  
+  const bool do_triu;
+  const bool do_tril;
   
   inline
   strip_trimat(const Op<T1, op_trimat>& X)
@@ -161,4 +164,68 @@ struct strip_trimat< Op<T1, op_trimat> >
 
 
 
+//
+
+
+
+template<typename T1>
+struct sp_strip_trans
+  {
+  typedef T1 stored_type;
+  
+  inline
+  sp_strip_trans(const T1& X)
+    : M(X)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  static constexpr bool do_htrans = false;
+  static constexpr bool do_strans = false;
+  
+  const T1& M;
+  };
+
+
+
+template<typename T1>
+struct sp_strip_trans< SpOp<T1, spop_htrans> >
+  {
+  typedef T1 stored_type;
+  
+  inline
+  sp_strip_trans(const SpOp<T1, spop_htrans>& X)
+    : M(X.m)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  static constexpr bool do_htrans = true;
+  static constexpr bool do_strans = false;
+  
+  const T1& M;
+  };
+
+
+
+template<typename T1>
+struct sp_strip_trans< SpOp<T1, spop_strans> >
+  {
+  typedef T1 stored_type;
+  
+  inline
+  sp_strip_trans(const SpOp<T1, spop_strans>& X)
+    : M(X.m)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  static constexpr bool do_htrans = false;
+  static constexpr bool do_strans = true;
+  
+  const T1& M;
+  };
+
+
+
 //! @}
