@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -22,7 +24,7 @@ template<typename eT>
 inline
 subview_cube<eT>::~subview_cube()
   {
-  arma_extra_debug_sigprint();
+  arma_extra_debug_sigprint_this(this);
   }
 
 
@@ -48,8 +50,223 @@ subview_cube<eT>::subview_cube
   , n_elem_slice(in_n_rows * in_n_cols)
   , n_slices    (in_n_slices)
   , n_elem      (n_elem_slice * in_n_slices)
+  {
+  arma_extra_debug_sigprint_this(this);
+  }
+
+
+
+template<typename eT>
+inline
+subview_cube<eT>::subview_cube(const subview_cube<eT>& in)
+  : m           (in.m           )
+  , aux_row1    (in.aux_row1    )
+  , aux_col1    (in.aux_col1    )
+  , aux_slice1  (in.aux_slice1  )
+  , n_rows      (in.n_rows      )
+  , n_cols      (in.n_cols      )
+  , n_elem_slice(in.n_elem_slice)
+  , n_slices    (in.n_slices    )
+  , n_elem      (in.n_elem      )
+  {
+  arma_extra_debug_sigprint(arma_str::format("this = %x   in = %x") % this % &in);
+  }
+
+
+
+template<typename eT>
+inline
+subview_cube<eT>::subview_cube(subview_cube<eT>&& in)
+  : m           (in.m           )
+  , aux_row1    (in.aux_row1    )
+  , aux_col1    (in.aux_col1    )
+  , aux_slice1  (in.aux_slice1  )
+  , n_rows      (in.n_rows      )
+  , n_cols      (in.n_cols      )
+  , n_elem_slice(in.n_elem_slice)
+  , n_slices    (in.n_slices    )
+  , n_elem      (in.n_elem      )
+  {
+  arma_extra_debug_sigprint(arma_str::format("this = %x   in = %x") % this % &in);
+  
+  // for paranoia
+  
+  access::rw(in.aux_row1    ) = 0;
+  access::rw(in.aux_col1    ) = 0;
+  access::rw(in.aux_slice1  ) = 0;
+  access::rw(in.n_rows      ) = 0;
+  access::rw(in.n_cols      ) = 0;
+  access::rw(in.n_elem_slice) = 0;
+  access::rw(in.n_slices    ) = 0;
+  access::rw(in.n_elem      ) = 0;
+  }
+
+
+
+template<typename eT>
+template<typename op_type>
+inline
+void
+subview_cube<eT>::inplace_op(const eT val)
   {
   arma_extra_debug_sigprint();
+  
+  subview_cube<eT>& t = *this;
+  
+  const uword t_n_rows   = t.n_rows;
+  const uword t_n_cols   = t.n_cols;
+  const uword t_n_slices = t.n_slices;
+  
+  for(uword s=0; s < t_n_slices; ++s)
+  for(uword c=0; c < t_n_cols;   ++c)
+    {
+    if(is_same_type<op_type, op_internal_plus >::yes)  { arrayops::inplace_plus ( slice_colptr(s,c), val, t_n_rows ); }
+    if(is_same_type<op_type, op_internal_minus>::yes)  { arrayops::inplace_minus( slice_colptr(s,c), val, t_n_rows ); }
+    if(is_same_type<op_type, op_internal_schur>::yes)  { arrayops::inplace_mul  ( slice_colptr(s,c), val, t_n_rows ); }
+    if(is_same_type<op_type, op_internal_div  >::yes)  { arrayops::inplace_div  ( slice_colptr(s,c), val, t_n_rows ); }
+    }
+  }
+
+
+
+
+
+
+template<typename eT>
+template<typename op_type, typename T1>
+inline
+void
+subview_cube<eT>::inplace_op(const BaseCube<eT,T1>& in, const char* identifier)
+  {
+  arma_extra_debug_sigprint();
+  
+  const ProxyCube<T1> P(in.get_ref());
+  
+  subview_cube<eT>& t = *this;
+  
+  const uword t_n_rows   = t.n_rows;
+  const uword t_n_cols   = t.n_cols;
+  const uword t_n_slices = t.n_slices;
+  
+  arma_debug_assert_same_size(t, P, identifier);
+  
+  const bool use_mp      = arma_config::openmp && ProxyCube<T1>::use_mp && mp_gate<eT>::eval(t.n_elem);
+  const bool has_overlap = P.has_overlap(t);
+  
+  if(has_overlap)  { arma_extra_debug_print("aliasing or overlap detected"); }
+  
+  if( (is_Cube<typename ProxyCube<T1>::stored_type>::value) || (use_mp) || (has_overlap) )
+    {
+    const unwrap_cube_check<typename ProxyCube<T1>::stored_type> tmp(P.Q, has_overlap);
+    const Cube<eT>& B = tmp.M;
+    
+    if( (is_same_type<op_type, op_internal_equ>::yes) && (t.aux_row1 == 0) && (t_n_rows == t.m.n_rows) )
+      {
+      for(uword s=0; s < t_n_slices; ++s)
+        {
+        arrayops::copy( t.slice_colptr(s,0), B.slice_colptr(s,0), t.n_elem_slice );
+        }
+      }
+    else
+      {
+      for(uword s=0; s < t_n_slices; ++s)
+      for(uword c=0; c < t_n_cols;   ++c)
+        {
+        if(is_same_type<op_type, op_internal_equ  >::yes)  { arrayops::copy         ( t.slice_colptr(s,c), B.slice_colptr(s,c), t_n_rows ); }
+        if(is_same_type<op_type, op_internal_plus >::yes)  { arrayops::inplace_plus ( t.slice_colptr(s,c), B.slice_colptr(s,c), t_n_rows ); }
+        if(is_same_type<op_type, op_internal_minus>::yes)  { arrayops::inplace_minus( t.slice_colptr(s,c), B.slice_colptr(s,c), t_n_rows ); }
+        if(is_same_type<op_type, op_internal_schur>::yes)  { arrayops::inplace_mul  ( t.slice_colptr(s,c), B.slice_colptr(s,c), t_n_rows ); }
+        if(is_same_type<op_type, op_internal_div  >::yes)  { arrayops::inplace_div  ( t.slice_colptr(s,c), B.slice_colptr(s,c), t_n_rows ); }
+        }
+      }
+    }
+  else  // use the Proxy
+    {
+    if(ProxyCube<T1>::use_at)
+      {
+      for(uword s=0; s < t_n_slices; ++s)
+      for(uword c=0; c < t_n_cols;   ++c)
+        {
+        eT* t_col_data = t.slice_colptr(s,c);
+        
+        for(uword r=0; r < t_n_rows; ++r)
+          {
+          const eT tmp = P.at(r,c,s);
+          
+          if(is_same_type<op_type, op_internal_equ  >::yes)  { (*t_col_data) =  tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_plus >::yes)  { (*t_col_data) += tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_minus>::yes)  { (*t_col_data) -= tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_schur>::yes)  { (*t_col_data) *= tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_div  >::yes)  { (*t_col_data) /= tmp; t_col_data++; }
+          }
+        }
+      }
+    else
+      {
+      typename ProxyCube<T1>::ea_type Pea = P.get_ea();
+      
+      uword count = 0;
+      
+      for(uword s=0; s < t_n_slices; ++s)
+      for(uword c=0; c < t_n_cols;   ++c)
+        {
+        eT* t_col_data = t.slice_colptr(s,c);
+        
+        for(uword r=0; r < t_n_rows; ++r)
+          {
+          const eT tmp = Pea[count];  count++;
+          
+          if(is_same_type<op_type, op_internal_equ  >::yes)  { (*t_col_data) =  tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_plus >::yes)  { (*t_col_data) += tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_minus>::yes)  { (*t_col_data) -= tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_schur>::yes)  { (*t_col_data) *= tmp; t_col_data++; }
+          if(is_same_type<op_type, op_internal_div  >::yes)  { (*t_col_data) /= tmp; t_col_data++; }
+          }
+        }
+      }
+    }
+  }
+
+
+
+template<typename eT>
+template<typename op_type>
+inline
+void
+subview_cube<eT>::inplace_op(const subview_cube<eT>& x, const char* identifier)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(check_overlap(x))
+    {
+    const Cube<eT> tmp(x);
+    
+    if(is_same_type<op_type, op_internal_equ  >::yes)  { (*this).operator= (tmp); }
+    if(is_same_type<op_type, op_internal_plus >::yes)  { (*this).operator+=(tmp); }
+    if(is_same_type<op_type, op_internal_minus>::yes)  { (*this).operator-=(tmp); }
+    if(is_same_type<op_type, op_internal_schur>::yes)  { (*this).operator%=(tmp); }
+    if(is_same_type<op_type, op_internal_div  >::yes)  { (*this).operator/=(tmp); }
+    
+    return;
+    }
+  
+  subview_cube<eT>& t = *this;
+  
+  arma_debug_assert_same_size(t, x, identifier);
+  
+  const uword t_n_rows   = t.n_rows;
+  const uword t_n_cols   = t.n_cols;
+  const uword t_n_slices = t.n_slices;
+  
+  for(uword s=0; s < t_n_slices; ++s)
+  for(uword c=0; c < t_n_cols;   ++c)
+    {
+    if(is_same_type<op_type, op_internal_equ  >::yes)  { arrayops::copy         ( t.slice_colptr(s,c), x.slice_colptr(s,c), t_n_rows ); }
+    if(is_same_type<op_type, op_internal_plus >::yes)  { arrayops::inplace_plus ( t.slice_colptr(s,c), x.slice_colptr(s,c), t_n_rows ); }
+    if(is_same_type<op_type, op_internal_minus>::yes)  { arrayops::inplace_minus( t.slice_colptr(s,c), x.slice_colptr(s,c), t_n_rows ); }
+    if(is_same_type<op_type, op_internal_schur>::yes)  { arrayops::inplace_mul  ( t.slice_colptr(s,c), x.slice_colptr(s,c), t_n_rows ); }
+    if(is_same_type<op_type, op_internal_div  >::yes)  { arrayops::inplace_div  ( t.slice_colptr(s,c), x.slice_colptr(s,c), t_n_rows ); }
+    }
   }
 
 
@@ -80,17 +297,7 @@ subview_cube<eT>::operator+= (const eT val)
   {
   arma_extra_debug_sigprint();
   
-  const uword local_n_rows   = n_rows;
-  const uword local_n_cols   = n_cols;
-  const uword local_n_slices = n_slices;
-  
-  for(uword slice = 0; slice < local_n_slices; ++slice)
-    {
-    for(uword col = 0; col < local_n_cols; ++col)
-      {
-      arrayops::inplace_plus( slice_colptr(slice,col), val, local_n_rows );
-      }
-    }
+  inplace_op<op_internal_plus>(val);
   }
 
 
@@ -102,17 +309,7 @@ subview_cube<eT>::operator-= (const eT val)
   {
   arma_extra_debug_sigprint();
   
-  const uword local_n_rows   = n_rows;
-  const uword local_n_cols   = n_cols;
-  const uword local_n_slices = n_slices;
-  
-  for(uword slice = 0; slice < local_n_slices; ++slice)
-    {
-    for(uword col = 0; col < local_n_cols; ++col)
-      {
-      arrayops::inplace_minus( slice_colptr(slice,col), val, local_n_rows );
-      }
-    }
+  inplace_op<op_internal_minus>(val);
   }
 
 
@@ -124,17 +321,7 @@ subview_cube<eT>::operator*= (const eT val)
   {
   arma_extra_debug_sigprint();
   
-  const uword local_n_rows   = n_rows;
-  const uword local_n_cols   = n_cols;
-  const uword local_n_slices = n_slices;
-  
-  for(uword slice = 0; slice < local_n_slices; ++slice)
-    {
-    for(uword col = 0; col < local_n_cols; ++col)
-      {
-      arrayops::inplace_mul( slice_colptr(slice,col), val, local_n_rows );
-      }
-    }
+  inplace_op<op_internal_schur>(val);
   }
 
 
@@ -146,17 +333,7 @@ subview_cube<eT>::operator/= (const eT val)
   {
   arma_extra_debug_sigprint();
   
-  const uword local_n_rows   = n_rows;
-  const uword local_n_cols   = n_cols;
-  const uword local_n_slices = n_slices;
-  
-  for(uword slice = 0; slice < local_n_slices; ++slice)
-    {
-    for(uword col = 0; col < local_n_cols; ++col)
-      {
-      arrayops::inplace_div( slice_colptr(slice,col), val, local_n_rows );
-      }
-    }
+  inplace_op<op_internal_div>(val);
   }
 
 
@@ -169,24 +346,7 @@ subview_cube<eT>::operator= (const BaseCube<eT,T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  const unwrap_cube<T1> tmp(in.get_ref());
-  
-  const Cube<eT>&         x = tmp.M;
-        subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "copy into subcube");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::copy( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_equ>(in, "copy into subcube");
   }
 
 
@@ -199,24 +359,7 @@ subview_cube<eT>::operator+= (const BaseCube<eT,T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  const unwrap_cube<T1> tmp(in.get_ref());
-  
-  const Cube<eT>&         x = tmp.M;
-        subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "addition");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_plus( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_plus>(in, "addition");
   }
 
 
@@ -229,24 +372,7 @@ subview_cube<eT>::operator-= (const BaseCube<eT,T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  const unwrap_cube<T1> tmp(in.get_ref());
-  
-  const Cube<eT>&         x = tmp.M;
-        subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "subtraction");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_minus( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_minus>(in, "subtraction");
   }
 
 
@@ -259,24 +385,7 @@ subview_cube<eT>::operator%= (const BaseCube<eT,T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  const unwrap_cube<T1> tmp(in.get_ref());
-  
-  const Cube<eT>&         x = tmp.M;
-        subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "element-wise multiplication");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_mul( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_schur>(in, "element-wise multiplication");
   }
 
 
@@ -289,24 +398,7 @@ subview_cube<eT>::operator/= (const BaseCube<eT,T1>& in)
   {
   arma_extra_debug_sigprint();
   
-  const unwrap_cube<T1> tmp(in.get_ref());
-  
-  const Cube<eT>&         x = tmp.M;
-        subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "element-wise division");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_div( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_div>(in, "element-wise division");
   }
 
 
@@ -319,30 +411,7 @@ subview_cube<eT>::operator= (const subview_cube<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(check_overlap(x))
-    {
-    const Cube<eT> tmp(x);
-    
-    (*this).operator=(tmp);
-    
-    return;
-    }
-  
-  subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "copy into subcube");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::copy( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_equ>(x, "copy into subcube");
   }
 
 
@@ -354,30 +423,7 @@ subview_cube<eT>::operator+= (const subview_cube<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(check_overlap(x))
-    {
-    const Cube<eT> tmp(x);
-    
-    (*this).operator+=(tmp);
-    
-    return;
-    }
-  
-  subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "addition");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_plus( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_plus>(x, "addition");
   }
 
 
@@ -389,30 +435,7 @@ subview_cube<eT>::operator-= (const subview_cube<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(check_overlap(x))
-    {
-    const Cube<eT> tmp(x);
-    
-    (*this).operator-=(tmp);
-    
-    return;
-    }
-  
-  subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "subtraction");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_minus( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_minus>(x, "subtraction");
   }
 
 
@@ -424,30 +447,7 @@ subview_cube<eT>::operator%= (const subview_cube<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(check_overlap(x))
-    {
-    const Cube<eT> tmp(x);
-    
-    (*this).operator%=(tmp);
-    
-    return;
-    }
-  
-  subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "element-wise multiplication");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_mul( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_schur>(x, "element-wise multiplication");
   }
 
 
@@ -459,30 +459,7 @@ subview_cube<eT>::operator/= (const subview_cube<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  if(check_overlap(x))
-    {
-    const Cube<eT> tmp(x);
-    
-    (*this).operator/=(tmp);
-    
-    return;
-    }
-  
-  subview_cube<eT>& t = *this;
-  
-  arma_debug_assert_same_size(t, x, "element-wise division");
-  
-  const uword t_n_rows   = t.n_rows;
-  const uword t_n_cols   = t.n_cols;
-  const uword t_n_slices = t.n_slices;
-  
-  for(uword slice = 0; slice < t_n_slices; ++slice)
-    {
-    for(uword col = 0; col < t_n_cols; ++col)
-      {
-      arrayops::inplace_div( t.slice_colptr(slice,col), x.slice_colptr(slice,col), t_n_rows );
-      }
-    }
+  inplace_op<op_internal_div>(x, "element-wise division");
   }
 
 
@@ -1134,59 +1111,55 @@ subview_cube<eT>::imbue(functor F)
 
 
 
-#if defined(ARMA_USE_CXX11)
+//! apply a lambda function to each slice, where each slice is interpreted as a matrix
+template<typename eT>
+inline
+void
+subview_cube<eT>::each_slice(const std::function< void(Mat<eT>&) >& F)
+  {
+  arma_extra_debug_sigprint();
   
-  //! apply a lambda function to each slice, where each slice is interpreted as a matrix
-  template<typename eT>
-  inline
-  void
-  subview_cube<eT>::each_slice(const std::function< void(Mat<eT>&) >& F)
+  Mat<eT> tmp1(n_rows, n_cols, arma_nozeros_indicator());
+  Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
+  
+  for(uword slice_id=0; slice_id < n_slices; ++slice_id)
     {
-    arma_extra_debug_sigprint();
-    
-    Mat<eT> tmp1(n_rows, n_cols);
-    Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
-    
-    for(uword slice_id=0; slice_id < n_slices; ++slice_id)
+    for(uword col_id=0; col_id < n_cols; ++col_id)
       {
-      for(uword col_id=0; col_id < n_cols; ++col_id)
-        {
-        arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
-        }
-      
-      F(tmp2);
-      
-      for(uword col_id=0; col_id < n_cols; ++col_id)
-        {
-        arrayops::copy( slice_colptr(slice_id, col_id), tmp1.colptr(col_id), n_rows );
-        }
+      arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
+      }
+    
+    F(tmp2);
+    
+    for(uword col_id=0; col_id < n_cols; ++col_id)
+      {
+      arrayops::copy( slice_colptr(slice_id, col_id), tmp1.colptr(col_id), n_rows );
       }
     }
+  }
+
+
+
+template<typename eT>
+inline
+void
+subview_cube<eT>::each_slice(const std::function< void(const Mat<eT>&) >& F) const
+  {
+  arma_extra_debug_sigprint();
   
+        Mat<eT> tmp1(n_rows, n_cols, arma_nozeros_indicator());
+  const Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
   
-  
-  template<typename eT>
-  inline
-  void
-  subview_cube<eT>::each_slice(const std::function< void(const Mat<eT>&) >& F) const
+  for(uword slice_id=0; slice_id < n_slices; ++slice_id)
     {
-    arma_extra_debug_sigprint();
-    
-          Mat<eT> tmp1(n_rows, n_cols);
-    const Mat<eT> tmp2('j', tmp1.memptr(), n_rows, n_cols);
-    
-    for(uword slice_id=0; slice_id < n_slices; ++slice_id)
+    for(uword col_id=0; col_id < n_cols; ++col_id)
       {
-      for(uword col_id=0; col_id < n_cols; ++col_id)
-        {
-        arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
-        }
-      
-      F(tmp2);
+      arrayops::copy( tmp1.colptr(col_id), slice_colptr(slice_id, col_id), n_rows );
       }
+    
+    F(tmp2);
     }
-  
-#endif
+  }
 
 
 
@@ -1234,6 +1207,38 @@ subview_cube<eT>::clean(const typename get_pod_type<eT>::result threshold)
 
 
 
+template<typename eT>
+inline
+void
+subview_cube<eT>::clamp(const eT min_val, const eT max_val)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(is_cx<eT>::no)
+    {
+    arma_debug_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "subview_cube::clamp(): min_val must be less than max_val" );
+    }
+  else
+    {
+    arma_debug_check( (access::tmp_real(min_val) > access::tmp_real(max_val)), "subview_cube::clamp(): real(min_val) must be less than real(max_val)" );
+    arma_debug_check( (access::tmp_imag(min_val) > access::tmp_imag(max_val)), "subview_cube::clamp(): imag(min_val) must be less than imag(max_val)" );
+    }
+  
+  const uword local_n_rows   = n_rows;
+  const uword local_n_cols   = n_cols;
+  const uword local_n_slices = n_slices;
+  
+  for(uword slice = 0; slice < local_n_slices; ++slice)
+    {
+    for(uword col = 0; col < local_n_cols; ++col)
+      {
+      arrayops::clamp( slice_colptr(slice,col), local_n_rows, min_val, max_val );
+      }
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -1336,12 +1341,13 @@ subview_cube<eT>::randn()
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::is_finite() const
   {
   arma_extra_debug_sigprint();
   
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "is_finite(): detection of non-finite values is not reliable in fast math mode"); }
+  
   const uword local_n_rows   = n_rows;
   const uword local_n_cols   = n_cols;
   const uword local_n_slices = n_slices;
@@ -1361,7 +1367,6 @@ subview_cube<eT>::is_finite() const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::is_zero(const typename get_pod_type<eT>::result tol) const
   {
@@ -1386,12 +1391,13 @@ subview_cube<eT>::is_zero(const typename get_pod_type<eT>::result tol) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::has_inf() const
   {
   arma_extra_debug_sigprint();
   
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_inf(): detection of non-finite values is not reliable in fast math mode"); }
+  
   const uword local_n_rows   = n_rows;
   const uword local_n_cols   = n_cols;
   const uword local_n_slices = n_slices;
@@ -1411,12 +1417,13 @@ subview_cube<eT>::has_inf() const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::has_nan() const
   {
   arma_extra_debug_sigprint();
   
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_nan(): detection of non-finite values is not reliable in fast math mode"); }
+  
   const uword local_n_rows   = n_rows;
   const uword local_n_cols   = n_cols;
   const uword local_n_slices = n_slices;
@@ -1434,6 +1441,32 @@ subview_cube<eT>::has_nan() const
 
 
 
+template<typename eT>
+inline
+bool
+subview_cube<eT>::has_nonfinite() const
+  {
+  arma_extra_debug_sigprint();
+  
+  if(arma_config::fast_math_warn)  { arma_debug_warn_level(1, "has_nonfinite(): detection of non-finite values is not reliable in fast math mode"); }
+  
+  const uword local_n_rows   = n_rows;
+  const uword local_n_cols   = n_cols;
+  const uword local_n_slices = n_slices;
+  
+  for(uword slice = 0; slice < local_n_slices; ++slice)
+    {
+    for(uword col = 0; col < local_n_cols; ++col)
+      {
+      if(arrayops::is_finite(slice_colptr(slice,col), local_n_rows) == false)  { return true; }
+      }
+    }
+  
+  return false;
+  }
+
+
+
 template<typename eT>
 inline
 eT
@@ -1487,7 +1520,7 @@ inline
 eT&
 subview_cube<eT>::operator()(const uword i)
   {
-  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
+  arma_debug_check_bounds( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
   
   const uword in_slice = i / n_elem_slice;
   const uword offset   = in_slice * n_elem_slice;
@@ -1508,7 +1541,7 @@ inline
 eT
 subview_cube<eT>::operator()(const uword i) const
   {
-  arma_debug_check( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
+  arma_debug_check_bounds( (i >= n_elem), "subview_cube::operator(): index out of bounds" );
   
   const uword in_slice = i / n_elem_slice;
   const uword offset   = in_slice * n_elem_slice;
@@ -1529,7 +1562,7 @@ arma_inline
 eT&
 subview_cube<eT>::operator()(const uword in_row, const uword in_col, const uword in_slice)
   {
-  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
+  arma_debug_check_bounds( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
   
   const uword index = (in_slice + aux_slice1)*m.n_elem_slice + (in_col + aux_col1)*m.n_rows + aux_row1 + in_row;
   
@@ -1543,7 +1576,7 @@ arma_inline
 eT
 subview_cube<eT>::operator()(const uword in_row, const uword in_col, const uword in_slice) const
   {
-  arma_debug_check( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
+  arma_debug_check_bounds( ( (in_row >= n_rows) || (in_col >= n_cols) || (in_slice >= n_slices) ), "subview_cube::operator(): location out of bounds" );
   
   const uword index = (in_slice + aux_slice1)*m.n_elem_slice + (in_col + aux_col1)*m.n_rows + aux_row1 + in_row;
   
@@ -1597,51 +1630,44 @@ subview_cube<eT>::slice_colptr(const uword in_slice, const uword in_col) const
 
 
 template<typename eT>
+template<typename eT2>
 inline
 bool
-subview_cube<eT>::check_overlap(const subview_cube<eT>& x) const
+subview_cube<eT>::check_overlap(const subview_cube<eT2>& x) const
   {
-  const subview_cube<eT>& t = *this;
+  if(is_same_type<eT,eT2>::value == false)  { return false; }
   
-  if(&t.m != &x.m)
-    {
-    return false;
-    }
-  else
-    {
-    if( (t.n_elem == 0) || (x.n_elem == 0) )
-      {
-      return false;
-      }
-    else
-      {
-      const uword t_row_start  = t.aux_row1;
-      const uword t_row_end_p1 = t_row_start + t.n_rows;
-      
-      const uword t_col_start  = t.aux_col1;
-      const uword t_col_end_p1 = t_col_start + t.n_cols;
-      
-      const uword t_slice_start  = t.aux_slice1;
-      const uword t_slice_end_p1 = t_slice_start + t.n_slices;
-      
-      
-      const uword x_row_start  = x.aux_row1;
-      const uword x_row_end_p1 = x_row_start + x.n_rows;
-      
-      const uword x_col_start  = x.aux_col1;
-      const uword x_col_end_p1 = x_col_start + x.n_cols;
-      
-      const uword x_slice_start  = x.aux_slice1;
-      const uword x_slice_end_p1 = x_slice_start + x.n_slices;
-      
-      
-      const bool outside_rows   = ( (x_row_start   >= t_row_end_p1  ) || (t_row_start   >= x_row_end_p1  ) );
-      const bool outside_cols   = ( (x_col_start   >= t_col_end_p1  ) || (t_col_start   >= x_col_end_p1  ) );
-      const bool outside_slices = ( (x_slice_start >= t_slice_end_p1) || (t_slice_start >= x_slice_end_p1) );
-      
-      return ( (outside_rows == false) && (outside_cols == false) && (outside_slices == false) );
-      }
-    }
+  const subview_cube<eT>& t = (*this);
+  
+  if(void_ptr(&(t.m)) != void_ptr(&(x.m)))  { return false; }
+  
+  if( (t.n_elem == 0) || (x.n_elem == 0) )  { return false; }
+  
+  const uword t_row_start  = t.aux_row1;
+  const uword t_row_end_p1 = t_row_start + t.n_rows;
+  
+  const uword t_col_start  = t.aux_col1;
+  const uword t_col_end_p1 = t_col_start + t.n_cols;
+  
+  const uword t_slice_start  = t.aux_slice1;
+  const uword t_slice_end_p1 = t_slice_start + t.n_slices;
+  
+  
+  const uword x_row_start  = x.aux_row1;
+  const uword x_row_end_p1 = x_row_start + x.n_rows;
+  
+  const uword x_col_start  = x.aux_col1;
+  const uword x_col_end_p1 = x_col_start + x.n_cols;
+  
+  const uword x_slice_start  = x.aux_slice1;
+  const uword x_slice_end_p1 = x_slice_start + x.n_slices;
+  
+  
+  const bool outside_rows   = ( (x_row_start   >= t_row_end_p1  ) || (t_row_start   >= x_row_end_p1  ) );
+  const bool outside_cols   = ( (x_col_start   >= t_col_end_p1  ) || (t_col_start   >= x_col_end_p1  ) );
+  const bool outside_slices = ( (x_slice_start >= t_slice_end_p1) || (t_slice_start >= x_slice_end_p1) );
+  
+  return ( (outside_rows == false) && (outside_cols == false) && (outside_slices == false) );
   }
 
 
@@ -1658,7 +1684,7 @@ subview_cube<eT>::check_overlap(const Mat<eT>& x) const
   
   for(uword slice = t_aux_slice1; slice < t_aux_slice2_plus_1; ++slice)
     {
-    if(t.m.mat_ptrs[slice] != NULL)
+    if(t.m.mat_ptrs[slice] != nullptr)
       {
       const Mat<eT>& y = *(t.m.mat_ptrs[slice]);
       
@@ -1686,15 +1712,22 @@ subview_cube<eT>::extract(Cube<eT>& out, const subview_cube<eT>& in)
   const uword n_cols   = in.n_cols;
   const uword n_slices = in.n_slices;
   
-  arma_extra_debug_print(arma_str::format("out.n_rows = %d   out.n_cols = %d    out.n_slices = %d    in.m.n_rows = %d   in.m.n_cols = %d   in.m.n_slices = %d") % out.n_rows % out.n_cols % out.n_slices % in.m.n_rows % in.m.n_cols % in.m.n_slices);
+  arma_extra_debug_print(arma_str::format("out.n_rows = %u   out.n_cols = %u    out.n_slices = %u    in.m.n_rows = %u   in.m.n_cols = %u   in.m.n_slices = %u") % out.n_rows % out.n_cols % out.n_slices % in.m.n_rows % in.m.n_cols % in.m.n_slices);
   
-  
-  for(uword slice = 0; slice < n_slices; ++slice)
+  if( (in.aux_row1 == 0) && (n_rows == in.m.n_rows) )
     {
-    for(uword col = 0; col < n_cols; ++col)
+    for(uword s=0; s < n_slices; ++s)
       {
-      arrayops::copy( out.slice_colptr(slice,col), in.slice_colptr(slice,col), n_rows );
+      arrayops::copy( out.slice_colptr(s,0), in.slice_colptr(s,0), in.n_elem_slice );
       }
+    
+    return;
+    }
+  
+  for(uword s=0; s < n_slices; ++s)
+  for(uword c=0; c < n_cols;   ++c)
+    {
+    arrayops::copy( out.slice_colptr(s,c), in.slice_colptr(s,c), n_rows );
     }
   }
 
@@ -2367,15 +2400,15 @@ subview_cube<eT>::cend() const
 template<typename eT>
 inline
 subview_cube<eT>::iterator::iterator()
-  : M            (NULL)
-  , current_ptr  (NULL)
-  , current_row  (0   )
-  , current_col  (0   )
-  , current_slice(0   )
-  , aux_row1     (0   )
-  , aux_col1     (0   )
-  , aux_row2_p1  (0   )
-  , aux_col2_p1  (0   )
+  : M            (nullptr)
+  , current_ptr  (nullptr)
+  , current_row  (0      )
+  , current_col  (0      )
+  , current_slice(0      )
+  , aux_row1     (0      )
+  , aux_col1     (0      )
+  , aux_row2_p1  (0      )
+  , aux_col2_p1  (0      )
   {
   arma_extra_debug_sigprint();
   // Technically this iterator is invalid (it does not point to a valid element)
@@ -2421,7 +2454,6 @@ subview_cube<eT>::iterator::iterator(subview_cube<eT>& in_sv, const uword in_row
 
 template<typename eT>
 inline
-arma_warn_unused
 eT&
 subview_cube<eT>::iterator::operator*()
   {
@@ -2462,7 +2494,6 @@ subview_cube<eT>::iterator::operator++()
 
 template<typename eT>
 inline
-arma_warn_unused
 typename subview_cube<eT>::iterator
 subview_cube<eT>::iterator::operator++(int)
   {
@@ -2477,7 +2508,6 @@ subview_cube<eT>::iterator::operator++(int)
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::iterator::operator==(const iterator& rhs) const
   {
@@ -2488,7 +2518,6 @@ subview_cube<eT>::iterator::operator==(const iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::iterator::operator!=(const iterator& rhs) const
   {
@@ -2499,7 +2528,6 @@ subview_cube<eT>::iterator::operator!=(const iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::iterator::operator==(const const_iterator& rhs) const
   {
@@ -2510,7 +2538,6 @@ subview_cube<eT>::iterator::operator==(const const_iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::iterator::operator!=(const const_iterator& rhs) const
   {
@@ -2528,8 +2555,8 @@ subview_cube<eT>::iterator::operator!=(const const_iterator& rhs) const
 template<typename eT>
 inline
 subview_cube<eT>::const_iterator::const_iterator()
-  : M            (NULL)
-  , current_ptr  (NULL)
+  : M            (nullptr)
+  , current_ptr  (nullptr)
   , current_row  (0   )
   , current_col  (0   )
   , current_slice(0   )
@@ -2600,7 +2627,6 @@ subview_cube<eT>::const_iterator::const_iterator(const subview_cube<eT>& in_sv,
 
 template<typename eT>
 inline
-arma_warn_unused
 const eT&
 subview_cube<eT>::const_iterator::operator*()
   {
@@ -2641,7 +2667,6 @@ subview_cube<eT>::const_iterator::operator++()
 
 template<typename eT>
 inline
-arma_warn_unused
 typename subview_cube<eT>::const_iterator
 subview_cube<eT>::const_iterator::operator++(int)
   {
@@ -2656,7 +2681,6 @@ subview_cube<eT>::const_iterator::operator++(int)
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::const_iterator::operator==(const iterator& rhs) const
   {
@@ -2667,7 +2691,6 @@ subview_cube<eT>::const_iterator::operator==(const iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::const_iterator::operator!=(const iterator& rhs) const
   {
@@ -2678,7 +2701,6 @@ subview_cube<eT>::const_iterator::operator!=(const iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::const_iterator::operator==(const const_iterator& rhs) const
   {
@@ -2689,7 +2711,6 @@ subview_cube<eT>::const_iterator::operator==(const const_iterator& rhs) const
 
 template<typename eT>
 inline
-arma_warn_unused
 bool
 subview_cube<eT>::const_iterator::operator!=(const const_iterator& rhs) const
   {
